var assert = require('assert'),
    suite = require('testingey').createSuite('Indexer Tests'),
    indexer = require('../lib/indexer'),
    stringIndex = indexer.create('StringIndex'),
    numericIndex = indexer.create({ name: 'NumericIndex', type: 'numeric' });

suite.setUp(function () {

  for (var i = 0; i < 10; ++i) {
    stringIndex.add('str-' + i, 'key-' + i);
    numericIndex.add(i, 'key-' + i);
  }

  // duplicate values
  for (var j = 0; j < 5; ++j) {
    stringIndex.add('str-' + i, 'key-' + (i + j));
    numericIndex.add(i, 'key-' + (i + j));
  }

});

suite.spec('Factory method', function () {
  assert.ok(stringIndex instanceof indexer.Indexer);
  assert.ok(numericIndex instanceof indexer.Indexer);
});

suite.spec('String lookup', function () {
  assert.equal(stringIndex.search('str-0')[0], 'key-0');
  assert.equal(stringIndex.search('str-10').length, 5);
  assert.equal(stringIndex.search('str-10', 0, 5).length, 0);
  assert.equal(stringIndex.search(['str-9', 'str-10']).length, 6);
});

suite.spec('Numeric lookups', function () {
  assert.equal(numericIndex.search(1)[0], 'key-1');
  assert.equal(numericIndex.search({ min: 5, max: 6 }).length, 2);
  assert.equal(numericIndex.search({ max: 3 }).length, 4);
});

suite.spec('All keys', function () {
  assert.equal(stringIndex.all().length, 15);
  assert.equal(numericIndex.all().length, 15);
  assert.equal(stringIndex.all(5).length, 5);
  assert.equal(numericIndex.all(5).length, 5);
  assert.equal(stringIndex.all(0, 5).length, 10);
  assert.equal(numericIndex.all(0, 5).length, 10);
});

suite.spec('Remove a value', function () {
  stringIndex.remove('str-0');
  numericIndex.remove(0);
  assert.equal(stringIndex.search('str-0').length, 0);
  assert.equal(numericIndex.search(0).length, 0);
});

suite.spec('Remove values by range', function () {
  numericIndex.add(101, 'key-101');
  numericIndex.add(102, 'key-102');
  numericIndex.add(103, 'key-103');
  numericIndex.remove({ min: 101, max: 103 });
  assert.equal(numericIndex.search({ min: 101, max: 103 }).length, 0);
});

suite.spec('Remove using a set of values', function () {
  stringIndex.add('addie', 'key-addie');
  stringIndex.add('sheryl', 'key-sheryl');
  stringIndex.add('jaydee', 'key-jaydee');
  stringIndex.remove(['sheryl', 'jaydee']);
  assert.equal(stringIndex.search('jaydee').length, 0);
});

suite.spec('Remove a key', function () {
  stringIndex.removeKey('key-10');
  numericIndex.removeKey('key-10');
  assert.equal(stringIndex.search('str-10').length, 4);
  assert.equal(numericIndex.search(10).length, 4);
});

suite.spec('Remove all data', function () {
  stringIndex.flush();
  numericIndex.flush();
  assert.equal(stringIndex.all().length, 0);
  assert.equal(numericIndex.all().length, 0);
});

suite.run();
